#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""This module contains InstanceGroup class."""

from yandex_cloud_client.base import YandexCloudObject


class InstanceGroup(YandexCloudObject):
    """This object represents a instance group."""

    pass