#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""This module contains RouteTable class."""

from yandex_cloud_client.base import YandexCloudObject


class RouteTable(YandexCloudObject):
    def __init__(self):
        pass
