#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""This module contains base endpoint urls."""

BASE_URL = 'https://api.cloud.yandex.net'
RESOURCE_MANAGER_URL = 'https://resource-manager.api.cloud.yandex.net'
IAM_URL = 'https://iam.api.cloud.yandex.net'
COMPUTE_URL = 'https://compute.api.cloud.yandex.net'
OPERATION_URL = 'https://operation.api.cloud.yandex.net'
CERTIFICATE_URL = 'https://certificate-manager.api.cloud.yandex.net'
CERTIFICATE_DATA_URL = 'https://data.certificate-manager.api.cloud.yandex.net'
