#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""This module contains Image class."""

from yandex_cloud_client.base import YandexCloudObject


class Image(YandexCloudObject):
    """This object represents a image."""

    def __init__(self, client=None, **kwargs):
        pass

    def create(self, *args, **kwargs):
        pass

    def delete(self, *args, **kwargs):
        pass

    def update(self, *args, **kwargs):
        pass

    def operations(self, *args, **kwargs):
        pass
