MIN_SHARED_INSTANCE_MEMORY = 524288
MIN_INSTANCE_MEMORY = 1048576
MAX_INSTANCE_MEMORY = 274877906944
MIN_DISK_SIZE = 4194304
MAX_DISK_SIZE = 4398046511104
INSTANCE_CORE_FRACTIONS = (0, 5, 20, 50, 100)
INSTANCE_CORES = (1,) + tuple(range(2, 34, 2))
INSTANCE_GPUS = (1,) + tuple(range(0, 6, 2))
DISK_MODES = ('READ_ONLY', 'READ_WRITE')
RESOURCE_ID_RE = '[a-z][a-z0-9-_]{,19}'
RESOURCE_NAME_RE = '|[a-z][-a-z0-9]{1,61}[a-z0-9]'
NETWORK_SETTINGS = ('STANDARD', 'SOFTWARE_ACCELERATED', 'HARDWARE_ACCELERATED')
INSTANCE_PLATFORMS = ('standard-v1', 'standard-v2')
IP_VERSIONS = ('IPV4', 'IPV6')
BASE_HEADERS = {'content-type': 'application/json'}
DEFAULT_TIMEOUT = 10
SECONDS_IN_DAY = 86400
AZ = ('ru-central1-a', 'ru-central1-b', 'ru-central1-c')
