#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""This module contains ApiKey, AccessKey and Key classes."""

from yandex_cloud_client.base import YandexCloudObject


class Key(YandexCloudObject):
    def __init__(self):
        pass


class ApiKey(YandexCloudObject):
    def __init__(self):
        pass


class AccessKey(YandexCloudObject):
    def __init__(self):
        pass
